/*Written with Emacs.*/
/*
  GtkDiskFree shows free space on your mounted partitions.  Copyright
  (C) 2001-2002 Dj-Death (Landwerlin Lionel)
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

#include "main.h"
#include "configure.h"
#include "interface.h"
#include "diskfree.h"
#include "pixmap.h"

gint last_width;

GdkPixbuf *
pixmap_draw_capacity (gint real, gint mounted, gint usedp, gint width)
{
	double x, y, max;
	gint height = 12;
	gchar *text;

	GdkPixbuf *pixbuf = NULL;

	cairo_text_extents_t extents;
	cairo_surface_t *surface;
	cairo_pattern_t *pattern;
	cairo_t *cr;
	
	max = width / 100.0;
	max = max * usedp;

	pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, width, height);
	surface = cairo_image_surface_create_for_data( gdk_pixbuf_get_pixels(pixbuf),
								CAIRO_FORMAT_RGB24, width, height,
								gdk_pixbuf_get_rowstride(pixbuf));
	cr = cairo_create(surface);

	/* Backgound color : white. */
	cairo_rectangle(cr, 0, 0, width - 1, height - 1);
	cairo_set_source_rgb(cr, 1, 1, 1);
	cairo_fill(cr);
	
	/* Foreground color. */
	if (!real || mounted == FS_UMOUNTED) {
		cairo_set_source_rgb(cr, 0.75, 0.75, 0.75);
		cairo_rectangle(cr, 0, 0, width, height);
		cairo_fill(cr);
	} else if (options->capacity_style[0]) {
		pattern = cairo_pattern_create_linear(0.0, 0.0, width, 0.0); 
		cairo_pattern_add_color_stop_rgba(pattern, 0, options->color1[2], options->color1[1], options->color1[0], 1);
		cairo_pattern_add_color_stop_rgba(pattern, 1, options->color2[2], options->color2[1], options->color2[0], 1);
		cairo_rectangle(cr, 0, 0, max, height - 1);
		cairo_set_source(cr, pattern);
		cairo_fill(cr);
		cairo_pattern_destroy(pattern);
	} else {
		if (usedp <= options->color_levels[0])
			cairo_set_source_rgb(cr, 0.0, 0.5, 0.0);
		else if (usedp <= options->color_levels[1])
			cairo_set_source_rgb(cr, 1.0, 0.5, 0.0);
		else
			cairo_set_source_rgb(cr, 1.0, 0.0, 0.0);
		cairo_rectangle(cr, 1, 1, max, height - 1);
		cairo_fill(cr);
	}
	
	/* Add text in capacity column ? */
	if (options->capacity_style[1]) {
		text = g_strdup_printf("%i %%", usedp);
		cairo_set_source_rgb(cr, options->color_text[2], options->color_text[1], options->color_text[0]);
		cairo_set_font_size(cr, 8);
		cairo_text_extents(cr, text, &extents);
		x = width / 2 - (extents.width / 2 + extents.x_bearing);
		y = height / 2 - (extents.height / 2 + extents.y_bearing);
		cairo_move_to(cr, x, y);
		cairo_show_text(cr, text);
		g_free(text);
	}
	
	cairo_rectangle(cr, 0, 0, width, height);
	cairo_set_source_rgb(cr, 0, 0, 0);
	cairo_set_line_width(cr, 1);
	cairo_stroke(cr);
	
	/* Free the allocated structures. */
	cairo_surface_destroy(surface);
	cairo_destroy(cr);
	
	return pixbuf;
}

void
list_make_pixbuf (filesystem_t *fs, gint width)
{
	fs->pixbuf = pixmap_draw_capacity(fs->real, fs->mounted, fs->pused, width);
	
	return;
}

void
gui_list_capacity_redraw (GtkTreeIter *iter, gint width)
{
	gint percent, status;
       	GdkPixbuf *pixbuf;
	
	gtk_tree_model_get(list_treemodel, iter, USEDP_PERCENT_COLUMN, &percent,
			   STATUS_COLUMN, &status, -1);
	pixbuf = pixmap_draw_capacity(status, status, percent, width);
	gtk_list_store_set(GTK_LIST_STORE(list_treemodel), iter, CAPACITY_COLUMN, pixbuf, -1);
	
	return;
}

gboolean
gui_list_capacitys_redraw (void)
{
	gint width;
	GtkTreeViewColumn *column;
	GtkTreeIter iter;
	
	/* Get size of the capacity column. */
	if (options->capacity_style[2]) {
		column = gtk_tree_view_get_column(GTK_TREE_VIEW(list_treeview), CAPACITY_COLUMN);
		width = gtk_tree_view_column_get_width(column);
		width -= 6;
	} else
		width = options->capacity_style[3];
	
	/* If the capacity column has same width than the last time we
	 * update it, also we do nothing. Same thing if width <= 0. */
	if ((last_width == width && !gui_color_update) || (width <= 0))
		return FALSE;
	
	if (!gtk_tree_model_get_iter_first(list_treemodel, &iter))
		return FALSE;
	
	do {
		gui_list_capacity_redraw(&iter, width);
	} while (gtk_tree_model_iter_next(list_treemodel, &iter));
       	
	last_width = width;
	gui_color_update = FALSE;
	
	return FALSE;
}

void
gui_list_capacitys_draw (GList *list)
{
	gint width;
	filesystem_t *fs;
	GtkTreeViewColumn *column;
	
	if (list == NULL)
		return;
	
	/* Get size of the capacity column. */
	if (options->capacity_style[2]) {
		column = gtk_tree_view_get_column(GTK_TREE_VIEW(list_treeview), CAPACITY_COLUMN);
		width = gtk_tree_view_column_get_width(column);
		width -= 6;
	} else
		width = options->capacity_style[3];
	
	/* If the capacity column has same width than the last time we
	 * update it, else we do nothing. Same thing if width <= 0. */
		
	list = g_list_first(list);
	while (list != NULL) {
		fs = FILESYSTEM(list->data);
		if (fs->need_update || gui_color_update)
			list_make_pixbuf(fs, width);
		list = list->next;
	}
	
	last_width = width;
	gui_color_update = FALSE;
	
	return;
}
